import java.util.*;
import java.io.DataInputStream;
import java.lang.Math;

import iBus.*;
import iBus.layers.*;
import iBus.exception.*;

import Ticker.*;

/* ***************************************************************************
 * Classe: Producer
 *
 */
public class Producer {
  static String appName = "Producer";
  static Log log = new Log(appName);		// Objecto log
  static long sleepTime = 2000;			// Tempo em ms entre cada tick
  static String channelVName = "Tick-Channel";
  static String channelVVer = "1.5";
  static String channelVSubject = "/P";
  static String channelVQoS = "Reliable";
  static String address;			// Endereco logico do editor

  public static void main(String argv[]) throws Exception {
    Application.setAppName(appName);

    /* ***********************************************************************
     * Instancia a stack para o canal volatil (V).
     *
     */

    // Pilha de protocolos para difusao fiavel (reliable multicast)
    final iBus.Stack stackV = new iBus.Stack(channelVQoS);

    // Aumenta os valores do timeout e heartbeat para tornar a deteccao de
    // falhas menos agressiva (valores em milisegundos). Por outro lado,
    // assumimos que este canal nao perde mensagens.
    REACH reachV = (REACH)stackV.getProtocolObject("REACH");
    reachV.setTimeout(60000);
    reachV.setHbeat(10000);
    NAK nakV = (NAK)stackV.getProtocolObject("NAK");
    nakV.setHBInterval(60000);

    iBusURL urlV = null;				// Endereco do canal V
    try {
      urlV = iBusURLFactory.create(channelVName, channelVVer, channelVSubject);
    }
    catch (Exception e) {
      // Coloca no log uma mensagem e termina a aplicacao (exit code=1)
      log.panic("main: iBusURLFactory.create failed: " + e);
    }

    // Define o endereco logico que permite identificar o produtor no canal
    address = urlV.getAddress();

    // Objecto (posting) a publicar no canal
    final Posting request = new Posting();
    Posting replies[] = null;

    // Conecta-se ao canal V como editor (provoca uma mudanca de vista em V)
    stackV.registerTalker(urlV);


    /* ***********************************************************************
     * Inicio da aplicacao
     */

    System.out.println(Application.getAppName() + ": inicio da aplicacao.\n");

    // Publica 100 mensagens no canal
    for (int count = 0; count < 100; count++) {
      System.out.print(count + "\t" + new Date());

      // Difunde o tick no canal para todos os assinantes activos
      try {
        replies = stackV.pull(urlV, request);
      }
      catch (iBusException e) {
        System.err.println("pull failed: " + e);
        System.exit(1);
      }

      System.out.println("\t" + new Date());

      // Suspende o thread durante algum tempo antes de enviar o proximo tick
      // Testes efectudados para valores de sleepTime iguais a
      // 2000, 1000, 500, 250, 100, 50 e 1 milisegundos
      Thread.currentThread().sleep(sleepTime);
      //executar o produtor sem threads de paragem

      // Em alternativa podemos usar enviar as mensagens manualmente
      //DataInputStream dis = new DataInputStream(System.in);
      //System.out.println("Pressione <ENTER> para enviar uma mensagem ");
      //dis.readLine();
    }
  }
}

// fim: Producer.java
